-- License CC0 (Creative Commons license) (c) darkfrei, 2022

love.window.setMode(1280, 800) -- Steam Deck resolution
Width, Height = love.graphics.getDimensions( )


function createBlocks (amount)
	for i = 1, amount do
		local w = math.random (10, 40)
		local h = math.random (10, 20)+math.random (10, 20)
		local x = math.random (Width/2-w)
		local y = math.random (Height-h)
		local vx = 0
		local vy = 10
		local block = {x=x, y=y, w=w, h=h, vx=vx, vy=vy, valid=true}
		table.insert(Blocks, block)
	end
end


function createBullets (amount)
	for i = 1, amount do
		local w = 10
		local h = 2
		local x = Width-2*w
		local y = math.random (Height-h)
		local vx = -300
		local vy = 0
		local bullet = {x=x, y=y, w=w, h=h, vx=vx, vy=vy, valid=true}
		table.insert(Bullets, bullet)
	end
end

function love.load()
	Blocks = {}
	createBlocks (40)
	
	Bullets = {}
	createBullets (10)
end

function CheckCollision(a, b)
	local x1,y1,w1,h1 = a.x,a.y,a.w,a.h
	local x2,y2,w2,h2 = b.x,b.y,b.w,b.h
	return 	x1 < x2+w2 and
			x2 < x1+w1 and
			y1 < y2+h2 and
			y2 < y1+h1
end

 
function love.update(dt)
	-- blocks
	for i, block in ipairs (Blocks) do
		block.x = block.x + dt*block.vx
		block.y = block.y + dt*block.vy
		if block.x < 0 or block.x+block.w > Width
			or block.y < 0 or block.y+block.h > Height then
			-- out of screen
			block.valid = false
		end
	end
	
	-- bullets
	for i, bullet in ipairs (Bullets) do
		bullet.x = bullet.x + dt*bullet.vx
		bullet.y = bullet.y + dt*bullet.vy
		if bullet.x < 0 or bullet.x+bullet.w > Width
			or bullet.y < 0 or bullet.y+bullet.h > Height then
			-- out of screen
			bullet.valid = false
		end
	end
	
	-- collisions
	for i, block in ipairs (Blocks) do
		if block.valid then
			for j, bullet in ipairs (Bullets) do
				if bullet.valid and CheckCollision (block, bullet) then
					block.valid = false
					bullet.valid = false
					break -- no other bullets
				end
			end
		end
	end
	
	-- cleaning
	-- blocks
	for i = #Blocks, 1, -1 do
		if not Blocks[i].valid then
			table.remove(Blocks, i)
			createBlocks (1)
		end
	end
	-- bullets
	for i = #Bullets, 1, -1 do
		if not Bullets[i].valid then
			table.remove(Bullets, i)
			createBullets (1)
		end
	end
end


function love.draw()
	-- blocks
	love.graphics.setColor(1,1,1)
	for i, block in ipairs (Blocks) do
		love.graphics.rectangle('line', block.x, block.y, block.w, block.h)
	end
	
	-- bullets
	love.graphics.setColor(1,1,0)
	for i, bullet in ipairs (Bullets) do
		love.graphics.rectangle('line', bullet.x, bullet.y, bullet.w, bullet.h)
	end
end
